import requests 
import random
from time import sleep
from loguru import logger


for _ in range(100):
    random_input = [[random.random() for _ in range(4)]]
    features = [f"feature_{i}" for i in range(4)]
    
    response = requests.get("http://127.0.0.1:80/predict/", 
                            json={"data": random_input, "features": features})
    logger.info(response.json())
    sleep(1)



from fastapi import FastAPI
from pydantic import BaseModel
from contextlib import asynccontextmanager
import pickle

import os
import pandas as pd
import sklearn
from loguru import logger
from typing import List
from tqdm import tqdm

from mlops_misis2025.utils import get_sql_connection


class InputResponse(BaseModel):
    data: List[List[float]]
    features: List[str]
    
class PredictResponse(BaseModel):
    output: float
    
    
def make_predict(data: InputResponse, model):
    data = pd.DataFrame(data.data, columns=data.features)
    pred = model.predict_proba(data)[:, 1]
    return PredictResponse(output=pred)


ml_models = {}
model_path = os.getenv("MODEL_PATH")


@asynccontextmanager
async def lifespan(app: FastAPI):
    global params
    with open(model_path, 'rb') as fin:
        ml_models["model"] = pickle.load(fin)
    yield
    ml_models.clear()


app = FastAPI(lifespan=lifespan)


@app.get("/")
def main():
    return "Entry Point"
    
        
@app.get("/predict/", response_model=PredictResponse)
def predict(request: InputResponse):
    return make_predict(request, ml_models['model'])



from pathlib import Path

from loguru import logger
from tqdm import tqdm
import typer

from mlops_misis2025.config import PROCESSED_DATA_DIR

app = typer.Typer()


@app.command()
def main(
    # ---- REPLACE DEFAULT PATHS AS APPROPRIATE ----
    input_path: Path = PROCESSED_DATA_DIR / "dataset.csv",
    output_path: Path = PROCESSED_DATA_DIR / "features.csv",
    # -----------------------------------------
):
    # ---- REPLACE THIS WITH YOUR OWN CODE ----
    logger.info("Generating features from dataset...")
    for i in tqdm(range(10), total=10):
        if i == 5:
            logger.info("Something happened for iteration 5.")
    logger.success("Features generation complete.")
    # -----------------------------------------


if __name__ == "__main__":
    app()



from pathlib import Path

import yaml
from dotenv import load_dotenv
from loguru import logger
from pydantic import BaseModel

# Load environment variables from .env file if it exists
load_dotenv()

# Configs
class ModelParams(BaseModel):
    model_type: str 

class SQLParams(BaseModel):
    username: str
    password: str
    database: str
    ip: str


class Config(BaseModel):
    random_state: int 
    model_params: ModelParams
    sql_params: SQLParams


def load_config(config_path) -> Config:
    with open(config_path) as fin:
        loaded_config_yaml = yaml.safe_load(fin)

    config = Config(**loaded_config_yaml)
    return config

# Paths
PROJ_ROOT = Path(__file__).resolve().parents[1]
logger.info(f"PROJ_ROOT path is: {PROJ_ROOT}")

DATA_DIR = PROJ_ROOT / "data"
RAW_DATA_DIR = DATA_DIR / "raw"
INTERIM_DATA_DIR = DATA_DIR / "interim"
PROCESSED_DATA_DIR = DATA_DIR / "processed"
EXTERNAL_DATA_DIR = DATA_DIR / "external"

MODELS_DIR = PROJ_ROOT / "models"

REPORTS_DIR = PROJ_ROOT / "reports"
FIGURES_DIR = REPORTS_DIR / "figures"
CONFIG_DIR = PROJ_ROOT / "configs"

# If tqdm is installed, configure loguru with tqdm.write
# https://github.com/Delgan/loguru/issues/135
try:
    from tqdm import tqdm

    logger.remove(0)
    logger.add(lambda msg: tqdm.write(msg, end=""), colorize=True)
except ModuleNotFoundError:
    pass



from pathlib import Path

from loguru import logger
import pandas as pd
from tqdm import tqdm
import typer
from sklearn.datasets import make_classification
from sklearn.model_selection import train_test_split


from mlops_misis2025.config import PROCESSED_DATA_DIR, RAW_DATA_DIR, CONFIG_DIR, load_config
from mlops_misis2025.utils import get_sql_connection

app = typer.Typer()


@app.command()
def main(
    new_arg: int = 5,
    input_path: Path = RAW_DATA_DIR / "dataset.csv",
    output_path: Path = PROCESSED_DATA_DIR / "dataset.csv",
    config_path: Path = CONFIG_DIR / "config.yaml",
):
    config = load_config(config_path)
    logger.info(f"{config=}")
    X, y = make_classification(n_samples=1000, n_features=4, random_state=config.random_state)
    X = pd.DataFrame(X, columns=[f"feature_{i+1}" for i in range(X.shape[1])])
    y = pd.DataFrame(y, columns=['target'])
    X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.1, random_state=config.random_state)

    conn = get_sql_connection(config.sql_params)
    X_train.to_sql(con=conn, name="X_train", if_exists='replace', index=False)
    X_test.to_sql(con=conn, name="X_test", if_exists='replace', index=False)
    y_train.to_sql(con=conn, name="y_train", if_exists='replace', index=False)
    y_test.to_sql(con=conn, name="y_test", if_exists='replace', index=False)

    y_test.to_csv(RAW_DATA_DIR / "y_test.csv", index=False)


if __name__ == "__main__":
    app()



import sqlalchemy

def get_sql_connection(params):  
    database_connection = sqlalchemy.create_engine('postgresql://{0}:{1}@{2}/{3}'.
                                               format(params.username, params.password, 
                                                      params.ip, params.database))
    return database_connection


from pathlib import Path

from loguru import logger
from tqdm import tqdm
import typer
import pandas as pd
from sklearn.metrics import roc_auc_score
import pickle

from mlops_misis2025.config import MODELS_DIR, PROCESSED_DATA_DIR, RAW_DATA_DIR, CONFIG_DIR, load_config
from mlops_misis2025.utils import get_sql_connection

app = typer.Typer()


@app.command()
def main(
    features_path: Path = PROCESSED_DATA_DIR / "test_features.csv",
    model_path: Path = MODELS_DIR / "model.pkl",
    predictions_path: Path = PROCESSED_DATA_DIR / "test_predictions.csv",
    config_path: Path = CONFIG_DIR / "config.yaml",
):
    config = load_config(config_path)
    conn = get_sql_connection(config.sql_params)
    X_test = pd.read_sql_table("X_test", conn)
    y_test = pd.read_sql_table("y_test", conn)
    output_df = X_test.copy()
    logger.info("loaded test data")

    with open(model_path, 'rb') as fin:
        model = pickle.load(fin)
    logger.info(f"loaded model from {model_path}")

    y_test_pred = model.predict_proba(X_test)[:, 1]
    logger.info(f"Scored new data")

    output_df['pred_proba'] = y_test_pred
    output_df.to_sql("batch_inference", conn)
    logger.info(f"Saved result to batch_inference table")


if __name__ == "__main__":
    app()



from pathlib import Path

import pandas as pd
from loguru import logger
from tqdm import tqdm
import typer
from sklearn.linear_model import LogisticRegression
import pickle

from mlops_misis2025.config import MODELS_DIR, PROCESSED_DATA_DIR, RAW_DATA_DIR, CONFIG_DIR, load_config
from mlops_misis2025.utils import get_sql_connection

app = typer.Typer()


@app.command()
def main(
    features_path: Path = PROCESSED_DATA_DIR / "features.csv",
    labels_path: Path = PROCESSED_DATA_DIR / "labels.csv",
    model_path: Path = MODELS_DIR / "model.pkl",
    config_path: Path = CONFIG_DIR / "config.yaml",
):
    config = load_config(config_path)
    conn = get_sql_connection(config.sql_params)
    X_train = pd.read_sql_table("X_train", conn)
    y_train = pd.read_sql_table("y_train", conn)
    logger.info("loaded train data")

    if config.model_params.model_type == "LogisticRegression":
        model = LogisticRegression(C=0.1)
        model.fit(X_train, y_train.values.ravel())
        logger.info("fit model")
    else:
        raise NotImplementedError

    with open(model_path, 'wb') as fin:
        pickle.dump(model, fin)
    logger.info(f"saved model to {model_path}")


if __name__ == "__main__":
    app()
